package org.hibernate.internal;

import org.hibernate.HibernateException;
import org.hibernate.SessionFactoryObserver;
import org.hibernate.cfg.Configuration;
import org.hibernate.cfg.Settings;
import org.hibernate.engine.spi.Mapping;
import org.hibernate.id.IdentifierGenerator;
import org.hibernate.mapping.PersistentClass;
import org.hibernate.mapping.RootClass;
import org.hibernate.persister.entity.EntityPersister;
import org.hibernate.persister.spi.PersisterFactory;
import org.hibernate.service.ServiceRegistry;

public class MutableSessionFactoryImpl
extends SessionFactoryImpl
{
   public MutableSessionFactoryImpl(final Configuration cfg,
                                    Mapping mapping,
                                    ServiceRegistry serviceRegistry,
                                    Settings settings,
                                    SessionFactoryObserver observer) throws HibernateException
   {
      super(cfg, mapping, serviceRegistry, settings, observer);
   }

   public void add(PersistentClass persistentClass, Configuration configuration)
   {

      IdentifierGenerator generator = persistentClass.getIdentifier().createIdentifierGenerator(
						configuration.getIdentifierGeneratorFactory(),
						getDialect(),
				      settings.getDefaultCatalogName(),
				      settings.getDefaultSchemaName(),
				      (RootClass) persistentClass
	   );
      identifierGenerators.put(persistentClass.getEntityName(), generator );

      EntityPersister entityPersister = serviceRegistry.getService(PersisterFactory.class).createEntityPersister(
                    persistentClass,
                    null, //TODO accessStrategy,
                    null, //TODO naturalIdAccessStrategy,
                    this,
                    configuration.getMapping()
      );
       entityPersisters.put(persistentClass.getEntityName(), entityPersister );
   }
}
