import com.goldencode.p2j.util.*;
import org.hibernate.*;
import org.hibernate.cfg.Configuration;
import org.hibernate.internal.MutableSessionFactoryImpl;
import org.hibernate.mapping.PersistentClass;
import org.hibernate.service.ServiceRegistry;
import org.hibernate.service.ServiceRegistryBuilder;

import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;

public class DynamicHibernate
{
   public static void main(String[] args)
   {
      DynamicHibernate dynamicHibernate = new DynamicHibernate();
      dynamicHibernate.execute();
   }

   private void execute()
   {
      try
      {
         Class.forName("org.h2.Driver");

         Connection connection = DriverManager.getConnection("jdbc:h2:mem:_temp;DB_CLOSE_DELAY=-1;MULTI_THREADED=1");
         Statement stmt = connection.createStatement();
         stmt.execute("create table tt (id integer, f1 bigint, f2 numeric, f3 boolean, f4 varchar, f5 date, f6 binary, f7 bigint, primary key (id))");
         stmt.execute("insert into tt (id, f1, f2, f3, f4, f5, f6, f7) values(1, 1, 1.1, false, 'RECORD 0', '2009-01-01', null, 1)");
         connection.close();

         // empty configuration
         Configuration configuration = new Configuration().configure("hibernate.h2.cfg.xml");

         // use our mutable session factory
         configuration.setProperty("session_factory.class", "org.hibernate.internal.MutableSessionFactoryImpl");

         ServiceRegistry serviceRegistry = new ServiceRegistryBuilder().applySettings(configuration.getProperties()).buildServiceRegistry();

         // build session factory, open session
         SessionFactory sessionFactory = configuration.buildSessionFactory(serviceRegistry);
         Session session = sessionFactory.openSession();

         // add entity to the already created session factory
         configuration.addFile(new File("Tt.hbm.xml"));
         configuration.buildMappings();
         PersistentClass pc = configuration.getClassMapping("Tt");
         ((MutableSessionFactoryImpl) sessionFactory).add(pc, configuration);

         Transaction tx = session.beginTransaction();
         Tt newTt = new Tt();
         newTt.setF1(new integer(5));
         newTt.setF2(new decimal(5.5));
         newTt.setF3(new logical(false));
         newTt.setF4(new character("RECORD 1"));
         newTt.setF5(new date(1, 1, 2009));
         newTt.setF6(new raw());
         newTt.setF7(new rowid((long) 5));
         session.save(newTt);
         tx.commit();

         tx = session.beginTransaction();
         newTt = new Tt();
         newTt.setF1(new integer(5));
         newTt.setF2(new decimal(5.5));
         newTt.setF3(new logical(true));
         newTt.setF4(new character("RECORD 2"));
         newTt.setF5(new date(1, 1, 2009));
         newTt.setF6(new raw());
         newTt.setF7(new rowid((long) 5));
         session.save(newTt);
         tx.commit();

         Query q1 = session.createSQLQuery("SELECT tt.* FROM TT tt").addEntity("tt", Tt.class);
         ScrollableResults res1 = q1.scroll();

         while (res1.next())
         {
            for (Object tt : res1.get())
            {
               System.out.println(tt);
            }
         }
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }
   }
}
